import cv2
import numpy as np
import argparse

roiPts = []
inputMode = False
saveROI = True

def selectROI(event, x, y, flags, param):
    global roiPts, inputMode

    if event == cv2.EVENT_LBUTTONDOWN:
        roiPts = [(x, y)]
        inputMode = True
    elif event == cv2.EVENT_LBUTTONUP:
    # record the ending (x, y) coordinates and indicate that
    # the cropping operation is finished
        roiPts.append((x, y))
        inputMode = False
            
            # draw a rectangle around the region of interest
        cv2.rectangle(image, roiPts[0], roiPts[1], (0, 255, 0), 2)
        cv2.imshow("image", image)

ap = argparse.ArgumentParser()
ap.add_argument("-v", "--video",
                help = "path to the video file")
ap.add_argument("-s","--save",
                help = "path to save the video file")
ap.add_argument("-r","--replace",
                help = "path to the replacing video")
args = vars(ap.parse_args())


cap = cv2.VideoCapture(args["video"])
if args.get("replace", False):
    saveROI = False
    cap2 = cv2.VideoCapture(args["replace"])

fourcc = cv2.VideoWriter_fourcc(*"MPEG")


ret, frame = cap.read()
ret2, frame2 = cap2.read()
image = frame.copy()
clone = image.copy()
cv2.namedWindow("image")
cv2.setMouseCallback("image", selectROI)

# keep looping until the 'q' key is pressed
while True:
    # display the image and wait for a keypress
    cv2.imshow("image", image)
    key = cv2.waitKey(1) & 0xFF
        
        # if the 'r' key is pressed, reset the cropping region
    if key == ord("r"):
        image = clone.copy()
    
        # if the 'c' key is pressed, break from the loop
    elif key == ord("c"):
        break

# if there are two reference points, then crop the region of interest
# from teh image and display it
if len(roiPts) == 2:
    roi = clone[roiPts[0][1]:roiPts[1][1], roiPts[0][0]:roiPts[1][0]]
    cv2.imshow("ROI", roi)
    
    r,h,c,w = roiPts[0][1], roiPts[1][1] - roiPts[0][1], roiPts[0][0], roiPts[1][0] - roiPts[0][0]
    track_window = (c,r,w,h)
# set up the ROI for tracking
hsv_roi =  cv2.cvtColor(roi, cv2.COLOR_BGR2HSV)
mask = cv2.inRange(hsv_roi, np.array((0., 60.,32.)), np.array((180.,255.,255.)))
roi_hist = cv2.calcHist([hsv_roi],[0],mask,[180],[0,180])
cv2.normalize(roi_hist,roi_hist,0,255,cv2.NORM_MINMAX)
# Setup the termination criteria, either 10 iteration or move by atleast 1 pt
term_crit = ( cv2.TERM_CRITERIA_EPS | cv2.TERM_CRITERIA_COUNT, 10, 1 )
writer = None
while True:
    ret ,frame = cap.read()
    ret2,frame2 = cap2.read()
    
    if ret is not True:
        break;
        
    hsv = cv2.cvtColor(frame, cv2.COLOR_BGR2HSV)
    dst = cv2.calcBackProject([hsv],[0],roi_hist,[0,180],1)
    # apply meanshift to get the new location
    ret, track_window = cv2.meanShift(dst, track_window, term_crit)
    # Draw it on image
    x,y,w,h = track_window
    f2 = frame.copy()
    img2 = cv2.rectangle(f2, (x,y), (x+w,y+h), (0,255,0), 2)
#    cv2.imshow('img2',img2)
#    cv2.imshow('roi',frame[y:y+h, x:x+w])
    if saveROI:
        if writer is None:
            fps = cap.get(cv2.CAP_PROP_FPS)
            writer = cv2.VideoWriter(args["save"], fourcc, int(fps),(w,h), True)
        output = np.zeros((h, w, 3), dtype="uint8")
        output = frame[y:y+h, x:x+w]
        writer.write(output)
    else:
        height = int(cap.get(cv2.CAP_PROP_FRAME_HEIGHT))
        width = int(cap.get(cv2.CAP_PROP_FRAME_WIDTH))
        height2 = int(cap2.get(cv2.CAP_PROP_FRAME_HEIGHT))
        width2 = int(cap2.get(cv2.CAP_PROP_FRAME_WIDTH))
        if writer is None:
            fps = cap.get(cv2.CAP_PROP_FPS)
            writer = cv2.VideoWriter(args["save"], fourcc, int(fps),(width,height), True)
#        output = np.zeros((height, width, 3), dtype="uint8")
        output = frame
        output[y:y+h, x:x+w] = 0
        if h < height2:
            h_s = h
        else:
            h_s = height2
        if w < width2:
            w_s = w
        else:
            w_s = width2
        output[y:y+h_s, x:x+w_s] = frame2[:h_s,:w_s]
        cv2.imshow('output',output)
        writer.write(output)
    k = cv2.waitKey(20) & 0xff
    if k == 27 or k == ord("q"):
        break
print(w,h)
print(width2,height2)
cv2.destroyAllWindows()
cap.release()
