# This is the main method of the model defining and training
from keras.layers import Convolution3D, MaxPooling3D, ZeroPadding3D
from keras.layers import Dense, Dropout, Activation, Flatten
from keras.layers import BatchNormalization
from keras.models import Sequential, load_model, Model
from keras.optimizers import SGD
from keras.utils import np_utils
import UCFSport5
import video_reader
import numpy as np
import os

batch_size = 4
nb_classes = 5
nb_epoch = 5
data_shape = (50,120,160,3)
piece_size = 80

model = Sequential()

model.add(ZeroPadding3D((1,1,1), input_shape=data_shape))
model.add(Convolution3D(32, (3, 3, 3), activation = 'relu'))
model.add(BatchNormalization(axis=1, momentum=0.99, epsilon=0.001))
model.add(ZeroPadding3D((1,1,1)))
model.add(Convolution3D(32, (3, 3, 3), activation = 'relu'))
model.add(BatchNormalization(axis=1, momentum=0.99, epsilon=0.001))
model.add(MaxPooling3D(pool_size=(2, 2, 2), strides=(2,2,2)))

model.add(ZeroPadding3D((1,1,1)))
model.add(Convolution3D(64, (3, 3, 3), activation = 'relu'))
model.add(BatchNormalization(axis=1, momentum=0.99, epsilon=0.001))
model.add(ZeroPadding3D((1,1,1)))
model.add(Convolution3D(64, (3, 3, 3), activation = 'relu'))
model.add(BatchNormalization(axis=1, momentum=0.99, epsilon=0.001))
model.add(MaxPooling3D(pool_size=(2, 2, 2), strides=(2,2,2)))

model.add(ZeroPadding3D((1,1,1)))
model.add(Convolution3D(128, (3, 3, 3), activation = 'relu'))
model.add(BatchNormalization(axis=1, momentum=0.99, epsilon=0.001))
model.add(ZeroPadding3D((1,1,1)))
model.add(Convolution3D(128, (3, 3, 3), activation = 'relu'))
model.add(BatchNormalization(axis=1, momentum=0.99, epsilon=0.001))
model.add(ZeroPadding3D((1,1,1)))
model.add(Convolution3D(128, (3, 3, 3), activation = 'relu'))
model.add(BatchNormalization(axis=1, momentum=0.99, epsilon=0.001))
model.add(MaxPooling3D(pool_size=(2, 2, 2), strides=(2,2,2)))

model.add(Flatten())
model.add(Dense(512, activation = 'relu'))
model.add(BatchNormalization(axis=1, momentum=0.99, epsilon=0.001))
model.add(Dropout(0.5))
model.add(Dense(nb_classes))
#model.add(Activation('softmax'))

# Let's train the model using SGD
sgd = SGD(lr=0.0001, decay=1e-6, momentum=0.9, nesterov=True)
model.compile(loss='categorical_crossentropy',
              optimizer=sgd,
              metrics=['accuracy'])

model.load_weights('model_weight.h5')

#(x_train, y_train), (x_test, y_test) = UCFSport5.load_data()
#
#videos = np.zeros((200, 50, 120, 160, 3), dtype = 'uint8')
#videos = videos.astype('float32')
#videos /= 255
#index = 0
#for s in x_test:
#    videos[index] = video_reader.load(s)
#    index += 1
#videos = videos[:index]
#
#result = model.predict(videos[120:])
#
#
#file = open('results10.txt','w')
#for j in result:
#    file.write(str(j[0]) + " ")
#    file.write(str(j[1]) + " ")
#    file.write(str(j[2]) + " ")
#    file.write(str(j[3]) + " ")
#    file.write(str(j[4]) + " ")
#    file.write('\n')
#file.close()

ori_video = video_reader.load('v_PushUps_g18_c01.avi')
blk_video = video_reader.load('pushups_block.avi')
rep_video = video_reader.load('push_hula.avi')
hla_video = video_reader.load('v_HulaHoop_g20_c05.avi')
videos = np.zeros((4,50,120,160,3))
videos[0] = ori_video
videos[1] = blk_video
videos[2] = rep_video
videos[3] = hla_video


result1 = model.predict(videos)
print(result1)

