# This is the main method of the model defining and training
from keras.layers import Convolution3D, MaxPooling3D, ZeroPadding3D
from keras.layers import Dense, Dropout, Activation, Flatten
from keras.layers import BatchNormalization
from keras.models import Sequential, load_model
from keras.optimizers import SGD
from keras.utils import np_utils
import UCFSport5
import video_reader
import numpy as np
import os

batch_size = 8
nb_classes = 5
nb_epoch = 1
data_shape = (50,120,160,3)
piece_size = 80

model = Sequential()

model.add(ZeroPadding3D((1,1,1), input_shape=data_shape))
model.add(Convolution3D(32, 3, 3, 3, activation = 'relu'))
model.add(BatchNormalization(axis=1, momentum=0.99, epsilon=0.001))
model.add(ZeroPadding3D((1,1,1)))
model.add(Convolution3D(32, 3, 3, 3, activation = 'relu'))
model.add(BatchNormalization(axis=1, momentum=0.99, epsilon=0.001))
model.add(MaxPooling3D(pool_size=(2, 2, 2), strides=(2,2,2)))

model.add(ZeroPadding3D((1,1,1)))
model.add(Convolution3D(64, 3, 3, 3, activation = 'relu'))
model.add(BatchNormalization(axis=1, momentum=0.99, epsilon=0.001))
model.add(ZeroPadding3D((1,1,1)))
model.add(Convolution3D(64, 3, 3, 3, activation = 'relu'))
model.add(BatchNormalization(axis=1, momentum=0.99, epsilon=0.001))
model.add(MaxPooling3D(pool_size=(2, 2, 2), strides=(2,2,2)))

model.add(ZeroPadding3D((1,1,1)))
model.add(Convolution3D(128, 3, 3, 3, activation = 'relu'))
model.add(BatchNormalization(axis=1, momentum=0.99, epsilon=0.001))
model.add(ZeroPadding3D((1,1,1)))
model.add(Convolution3D(128, 3, 3, 3, activation = 'relu'))
model.add(BatchNormalization(axis=1, momentum=0.99, epsilon=0.001))
model.add(ZeroPadding3D((1,1,1)))
model.add(Convolution3D(128, 3, 3, 3, activation = 'relu'))
model.add(BatchNormalization(axis=1, momentum=0.99, epsilon=0.001))
model.add(MaxPooling3D(pool_size=(2, 2, 2), strides=(2,2,2)))

model.add(Flatten())
model.add(Dense(512, activation = 'relu'))
model.add(Dropout(0.5))
model.add(Dense(nb_classes))
model.add(Activation('softmax'))

# Let's train the model using SGD
sgd = SGD(lr=0.0001, decay=1e-6, momentum=0.9, nesterov=True)
model.compile(loss='categorical_crossentropy',
              optimizer=sgd,
              metrics=['accuracy'])


# The data, shuffled and split between train and test sets:
(x_train, y_train), (x_test, y_test) = UCFSport5.load_data()

nb_piece = int(len(x_train)*1.0/piece_size)
if len(x_train)%piece_size != 0:
	nb_piece += 1

for i in range(0,nb_piece):
	if (i+1)*piece_size > len(x_train) or (i+1)*(piece_size/4) > len(x_test):
		x_train_s = x_train[i*piece_size:]
		y_train_s = y_train[i*piece_size:]
		x_test_s = x_test[i*(piece_size/4):]
		y_test_s = y_test[i*(piece_size/4):]
		X_train_s = np.zeros((len(x_train_s), 50, 120, 160, 3),dtype = "uint8")
		X_test_s = np.zeros((len(x_test_s), 50, 120, 160, 3),dtype = "uint8")
	else:
		x_train_s = x_train[i*piece_size:(i+1)*piece_size]
		y_train_s = y_train[i*piece_size:(i+1)*piece_size]
		x_test_s = x_test[i*int(piece_size/4):(i+1)*int(piece_size/4)]
		y_test_s = y_test[i*int(piece_size/4):(i+1)*int(piece_size/4)]
		X_train_s = np.zeros((piece_size, 50, 120, 160, 3),dtype = "uint8")
		X_test_s = np.zeros((int(piece_size/4), 50, 120, 160, 3),dtype = "uint8")

	j = 0
	for video in x_train_s:
		X_train_s[j] = video_reader.load(video)
		j += 1

	
	j = 0
	for video in x_test_s:
		X_test_s[j] = video_reader.load(video)
		j += 1

	# Convert class vectors to binary class matrices.
	Y_train_s = np_utils.to_categorical(y_train_s, nb_classes)
	Y_test_s = np_utils.to_categorical(y_test_s, nb_classes)


	X_train_s = X_train_s.astype('float32')
	X_test_s = X_test_s.astype('float32')
	X_train_s /= 255
	X_test_s /= 255

	model.fit(X_train_s, Y_train_s,
		      batch_size=batch_size,
		      nb_epoch=nb_epoch,
		      validation_data=(X_test_s, Y_test_s),
		      shuffle=True)


model.save('model.h5')
model.save_weights('model_weight.h5')


